/*
Copyright  2012 DMD, LLC.  Government Purpose Rights, ALL OTHER RIGHTS RESERVED
This computer software was developed pursuant to Contract Number W9113M-08-D-0003-0007 with the US
Government. Unauthorized Use, Reproduction, Modification, or Disclosure is strictly prohibited.
*/
// Convert a column name to its XML attribute name and get the value of that
// named attribute.
// E.g., "Column Name" becomes "ows_Column_x0020_Name".
$.fn.xattr = function(name) {
	return this.attr("ows_" + name.replace(/ /g, "_x0020_").replace(/\//g, "_x002f_"));
};

// Get the numeric ID from a lookup value.
// E.g., "27;#Item Title" returns 27.
var getIdFromLookup = function(lookup) {
	return lookup.indexOf(";") ? Number(lookup.substring(0, lookup.indexOf(";"))) : undefined;
};

var getDisplayFromLookup = function (lookup) {
    return lookup.indexOf("#") ? lookup.substring(lookup.indexOf("#") + 1) : undefined;
};

var encodeTextForSoap = function (text) {
    //   return text.replace("\"", "&quot;").replace("<", "&lt;").replace(">", "&gt;").replace("&", "&amp;");
    //   return text.replace(/&/gi, "&amp;").replace(/"/gi, "&quot;").replace(/'/gi, "&apos;").replace(/</gi, "&lt;").replace(/>/gi, "&gt;");
    //   return text.replace(/"/gi, "&quot;").replace(/'/gi, "&apos;").replace(/</gi, "&lt;").replace(/>/gi, "&gt;").replace(/&/gi, "&amp;");
    return text.replace(/"/gi, "&quot;").replace(/</gi, "&lt;").replace(/>/gi, "&gt;").replace(/&/gi, "&amp;");
};

var formatCurrency = function (num) {
    num = isNaN(num) || num === '' || num === null ? 0.00 : num;
    return parseFloat(num).toFixed(2);
};

// Truncate time from date time value
var getDateWithoutTime = function(dateTime) {
    return dateTime.split(" ")[0];
};

// determines whether an object is empty i.e. contains no properties
var isEmpty = function (obj) {
    for (var prop in obj) {
        if (obj.hasOwnProperty(prop))
            return false;
    }
    return true;
};

var sws = function () {
    var wrap = function (name, attributes, value) {
        if (typeof attributes === "string") {
            value = attributes;
            attributes = undefined;
        }

        if (attributes && attributes.length) {
            attributes = $.map(attributes, function (attribute) {
                return attribute.name + "='" + attribute.value + "'";
            });

            attributes = " " + attributes.join(" ");
        }
        else {
            attributes = "";
        }

        if (value !== undefined) {
            return "<" + name + attributes + ">" + value + "</" + name + ">";
        }
        else {
            return "<" + name + attributes + " />";
        }
    };

    var _serviceParameters = [];

    _serviceParameters["Webs"] = { url: "Webs.asmx", namespace: "http://schemas.microsoft.com/sharepoint/soap/" };
    _serviceParameters["Lists"] = { url: "Lists.asmx", namespace: "http://schemas.microsoft.com/sharepoint/soap/" };
    _serviceParameters["People"] = { url: "People.asmx", namespace: "http://schemas.microsoft.com/sharepoint/soap/" };

    var _services = [];

    _services["SearchPrincipals"] = {
        method: "SearchPrincipals",
        url: _serviceParameters["People"].url,
        namespace: _serviceParameters["People"].namespace,
        validate: function (options) {
            return options !== undefined && options.principalType !== undefined;
        },
        createElements: function (options) {
            var elements = [];

            elements.push(wrap("searchText", "" + (options.searchText || "\\")));
            elements.push(wrap("maxResults", "" + (options.maxResults || 2000000000)));
            elements.push(wrap("principalType", "" + (options.principalType || "All")));

            return elements;
        }
    };

    _services["GetWeb"] = {
        method: "GetWeb",
        url: _serviceParameters["Webs"].url,
        namespace: _serviceParameters["Webs"].namespace,
        validate: function (options) {
            return options !== undefined && options.webUrl !== undefined;
        },
        createElements: function (options) {
            return [wrap("webUrl", options.webUrl)];
        }
    };

    _services["GetWebCollection"] = {
        method: "GetWebCollection",
        url: _serviceParameters["Webs"].url,
        namespace: _serviceParameters["Webs"].namespace,
        validate: function (options) {
            return options !== undefined;
        },
        createElements: function (options) {
            return [];
        }
    };

    _services["GetList"] = {
        method: "GetList",
        url: _serviceParameters["Lists"].url,
        namespace: _serviceParameters["Lists"].namespace,
        validate: function (options) {
            return options !== undefined && options.listName !== undefined;
        },
        createElements: function (options) {
            return [wrap("listName", options.listName)];
        }
    };

    _services["GetListItems"] = {
        method: "GetListItems",
        url: _serviceParameters["Lists"].url,
        namespace: _serviceParameters["Lists"].namespace,
        validate: function (options) {
            return options !== undefined && options.listName !== undefined;
        },
        createElements: function (options) {

            var elements = [wrap("listName", options.listName)];

            if (options.query) {
                elements.push(wrap("query", wrap("Query", options.query)));
            }

            var queryOptions = [];

            if (options.folder) {
                queryOptions.push(wrap("Folder", options.folder));
            }

            if (options.fieldNames) {
                queryOptions.push(wrap("IncludeMandatoryColumns", "False"));

                var fieldNames = $.map(options.fieldNames, function (fieldName) {
                    return wrap("FieldRef", [{ name: "Name", value: fieldName.replace(/ /g, "_x0020_")}]);
                });

                elements.push(wrap("viewFields", wrap("ViewFields", fieldNames.join(""))));
            }

            if (options.recursive) {
                queryOptions.push(wrap("ViewAttributes", [{ name: "Scope", value: "Recursive"}]));
            }

            if (options.recursiveAll) {
                queryOptions.push(wrap("ViewAttributes", [{ name: "Scope", value: "RecursiveAll"}]));
            }

            if (options.filesOnly) {
                queryOptions.push(wrap("ViewAttributes", [{ name: "Scope", value: "FilesOnly"}]));
            }

            if (queryOptions.length) {
                elements.push(wrap("queryOptions", wrap("QueryOptions", queryOptions.join(""))));
            }

            elements.push(wrap("rowLimit", "" + (options.rowLimit || 0)));

            return elements;
        }
    };

    _services["UpdateListItems"] = {
        method: "UpdateListItems",
        url: _serviceParameters["Lists"].url,
        namespace: _serviceParameters["Lists"].namespace,
        validate: function (options) {
            var allHaveId = true;
            $.each(options.items, function (n, item) {
                allHaveId &= (item.ID !== undefined);
            });


            return options !== undefined && options.listName !== undefined && allHaveId;
        },
        createElements: function (options) {
            var elements = [wrap("listName", options.listName)],
				methods = [];
            $.each(options.items, function (n, item) {
                var method = [];

                for (var key in item) {
                    method.push(wrap("Field", [{ name: "Name", value: key}], item[key]));
                }

                methods.push(wrap("Method", [{ name: "ID", value: methods.length + 1 }, { name: "Cmd", value: options.command || "Update"}], method.join("")));
            });

            var batchAttributes = [{ name: "OnError", value: "Continue"}];
            if (options !== undefined && options.rootFolder !== undefined) {
                batchAttributes.push({ name: "RootFolder", value: options.rootFolder });
            }
            elements.push(wrap("updates", wrap("Batch", batchAttributes, methods.join(""))));

            return elements;
        }
    };

    // Generate the SOAP XML document that constitutes the request.
    var _createEnvelope = function (options) {
        var service = _services[options.service];

        if (service === undefined || !service.validate(options)) {
            return;
        }

        var body = wrap(
			"soap:Envelope",
			[{ name: "xmlns:soap", value: "http://schemas.xmlsoap.org/soap/envelope/"}],
			wrap(
				"soap:Body",
				wrap(
					service.method,
					[{ name: "xmlns", value: service.namespace}],
					service.createElements(options).join("")
				)//
			)
		);

        return {
            soapActionHeader: service.namespace + service.method,
            url: service.url,
            message: "<?xml version='1.0' encoding='UTF-8'?>" + body
        };
    };

    return {
        request: function (options) {
            var envelope = _createEnvelope(options);
            if (envelope === undefined) {
                return;
            }
            var fullPath = $().SPServices.SPGetCurrentSite() + "/_vti_bin/";
            //var temp = (options.folder || ".") + "/_vti_bin/" + envelope.url; -- original
            // var temp = (options.folder || "") + "/sites/PRTracker/_vti_bin/" + envelope.url;
            var temp = fullPath + envelope.url;

            $.ajax($.extend(
				options.ajaxOptions || {},
				{
				    beforeSend: function (xhr) {
				        xhr.setRequestHeader("SOAPAction", envelope.soapActionHeader);
				    },
				    url: temp,
				    contentType: "text/xml; charset='utf-8'",
				    type: "post",
				    dataType: options.dataType || "xml",
				    cache: false,
				    data: envelope.message
				}
			));
        }
    };
} ();
